@@ -149,11 +149,17 @@ impl FixedBitSet
     /// Use `..` to count the whole content of the bitset.
     ///
     /// **Panics** if the range extends past the end of the bitset.
+    #[inline]
     pub fn count_ones<T: IndexRange>(&self, range: T) -> usize
     {
         let start = range.start().unwrap_or(0);
         let end = range.end().unwrap_or(self.length);
         assert!(start <= end && end <= self.length);
+        self.count_ones_impl(start, end)
+    }
+
+    #[inline]
+    fn count_ones_impl(&self, start: usize, end: usize) -> usize {
         let (first_block, first_rem) = div_rem(start, BITS);
         let (last_block, last_rem) = div_rem(end, BITS);
         let mut sum = 0usize;
@@ -165,8 +171,9 @@ impl FixedBitSet
         let mask = |x| if x != 0 { Block::max_value() >> (BITS - x) } else { 0 };
         let mask_first_block: Block = mask(first_rem);
         let mask_last_block: Block = mask(last_rem);
-        sum += (self.data[last_block] & mask_last_block).count_ones() as usize;
-        sum -= (self.data[first_block] & mask_first_block).count_ones() as usize;
+        let get_or_0 = |i| self.data.get(i).map_or(0, |&x| x);
+        sum += (get_or_0(last_block) & mask_last_block).count_ones() as usize;
+        sum -= (get_or_0(first_block) & mask_first_block).count_ones() as usize;
         sum
     }
 
@@ -184,6 +191,77 @@ impl FixedBitSet
     {
         &mut self.data
     }
+
+    /// Iterates over all enabled bits.
+    ///
+    /// Iterator element is the index of the `1` bit, type `usize`.
+    #[inline]
+    pub fn ones(&self) -> Ones {
+        match self.as_slice().split_first() {
+            Some((&block, rem)) => {
+                Ones {
+                    current_bit_idx: 0,
+                    current_block_idx: 0,
+                    current_block: block,
+                    remaining_blocks: rem
+                }
+            }
+            None => {
+                Ones {
+                    current_bit_idx: 0,
+                    current_block_idx: 0,
+                    current_block: 0,
+                    remaining_blocks: &[]
+                }
+            }
+        }
+    }
+}
+
+pub struct Ones<'a> {
+    current_bit_idx: usize,
+    current_block_idx: usize,
+    remaining_blocks: &'a [Block],
+    current_block: Block
+}
+
+impl<'a> Iterator for Ones<'a> {
+    type Item = usize; // the bit position of the '1'
+
+    fn next(&mut self) -> Option<Self::Item> {
+        let mut block = self.current_block;
+        let mut idx = self.current_bit_idx;
+
+        loop {
+            loop {
+                if (block & 1) == 1 {
+                    self.current_block = block >> 1;
+                    self.current_bit_idx = idx + 1;
+                    return Some(idx);
+                }
+                // reordering the two lines below makes a huge (2x) difference in performance!
+                block = block >> 1;
+                idx += 1;
+                if block == 0 {
+                    break;
+                }
+            }
+
+            // go to next block
+            match self.remaining_blocks.split_first() {
+                Some((&next_block, rest)) => {
+                    self.remaining_blocks = rest;
+                    self.current_block_idx += 1;
+                    idx = self.current_block_idx * BITS;
+                    block = next_block;
+                }
+                None => {
+                    // last block => done
+                    return None;
+                }
+            }
+        }
+    }
 }
 
 impl Clone for FixedBitSet
@@ -309,6 +387,43 @@ fn count_ones() {
     assert_eq!(fb.count_ones(100..100), 0);
 }
 
+#[test]
+fn ones() {
+    let mut fb = FixedBitSet::with_capacity(100);
+    fb.set(11, true);
+    fb.set(12, true);
+    fb.set(7, true);
+    fb.set(35, true);
+    fb.set(40, true);
+    fb.set(77, true);
+    fb.set(95, true);
+    fb.set(50, true);
+    fb.set(99, true);
+
+    let ones: Vec<_> = fb.ones().collect();
+
+    assert_eq!(vec![7, 11, 12, 35, 40, 50, 77, 95, 99], ones);
+}
+
+#[test]
+fn iter_ones_range() {
+    fn test_range(from: usize, to: usize, capa: usize) {
+        assert!(to <= capa);
+        let mut fb = FixedBitSet::with_capacity(capa);
+        for i in from..to {
+            fb.insert(i);
+        }
+        let ones: Vec<_> = fb.ones().collect();
+        let expected: Vec<_> = (from..to).collect();
+        assert_eq!(expected, ones);
+    }
+
+    for i in 0..100 {
+      test_range(i, 100, 100);
+      test_range(0, i, 100);
+    }
+}
+
 #[should_panic]
 #[test]
 fn count_ones_oob() {
@@ -323,8 +438,25 @@ fn count_ones_negative_range() {
     fb.count_ones(90..80);
 }
 
+#[test]
+fn count_ones_panic() {
+    for i in 1..128 {
+        let fb = FixedBitSet::with_capacity(i);
+        for j in 0..fb.len() + 1 {
+            for k in j..fb.len() + 1 {
+                assert_eq!(fb.count_ones(j..k), 0);
+            }
+        }
+    }
+}
+
+
 #[test]
 fn default() {
     let fb = FixedBitSet::default();
     assert_eq!(fb.len(), 0);
 }
+
+
+
+
